@@ -1,10 +1,16 @@
+# Cache::Memcached::AnyEvent
+#   is the API
+# Cache::Memcached::AnyEvent::Selector
+#   is the guts that selects sockets to talk to.
+# Cache::Memcached::AnyEvent::Protocol
+#   is the guy that actually speaks to memcached
+
 package Cache::Memcached::AnyEvent;
 use strict;
 use AnyEvent;
 use AnyEvent::Handle;
 use AnyEvent::Socket;
 use Carp;
-use String::CRC32;
 use Storable ();
 
 use constant +{
@@ -14,20 +20,22 @@ use constant +{
     COMPRESS_SAVINGS => 0.20,
 };
 
-our $VERSION = '0.00018';
+our $VERSION = '0.00019';
 
 sub new {
     my $class = shift;
+    my %args  = @_ == 1 ? %{$_[0]} : @_;
+
+    my $protocol_class = delete $args{protocol_class} || 'Text';
+    my $selector_class = delete $args{selector_class} || 'Traditional';
     my $self  = bless {
         auto_reconnect => 5,
-        compess_threshold => 10_000,
-        hash_cb => \&_modulo_hasher,
+        compress_threshold => 10_000,
         protocol => undef,
-        protocol_class => 'Text',
         reconnect_delay => 5,
         servers => undef,
         namespace => undef,
-        @_ == 1 ? %{$_[0]} : @_,
+        %args,
         _active_servers => [],
         _active_server_count => 0,
         _is_connected => undef,
@@ -36,27 +44,33 @@ sub new {
         _server_handles => undef,
     }, $class;
 
-    $self->{protocol} ||= $self->_build_protocol;
+    $self->{selector} ||= $self->_build_selector( $selector_class );
+    $self->{protocol} ||= $self->_build_protocol( $protocol_class );
 
     return $self;
 }
 
+sub _build_selector {
+    $_[0]->_build_helper( 'Cache::Memcached::AnyEvent::Selector', $_[1] );
+}
 sub _build_protocol {
-    my $self = shift;
-    my $p_class = $self->{protocol_class};
-    if ($p_class !~ s/^\+//) {
-        $p_class = "Cache::Memcached::AnyEvent::Protocol::$p_class";
+    $_[0]->_build_helper( 'Cache::Memcached::AnyEvent::Protocol', $_[1] );
+}
+sub _build_helper {
+    my ($self, $prefix, $klass) = @_;
+    if ($klass !~ s/^\+//) {
+        $klass = "${prefix}::$klass";
     }
 
-    $p_class =~ s/[^\w:_]//g; # cleanse
+    $klass =~ s/[^\w:_]//g; # cleanse
 
-    eval "require $p_class";
+    eval "require $klass";
     Carp::confess $@ if $@;
-    return $p_class->new(memcached => $self);
+    return $klass->new(memcached => $self);
 }
 
 BEGIN {
-    foreach my $attr qw(auto_reconncet compress_threshold reconnect_delay servers namespace) {
+    foreach my $attr qw(auto_reconnect compress_threshold reconnect_delay servers namespace) {
         eval <<EOSUB;
             sub $attr {
                 my \$self = shift;
@@ -78,22 +92,22 @@ sub protocol {
         my $obj = shift;
         my $class = ref $obj;
         $self->{protocol} = $obj;
-        $self->{protocol_class} = $class;
     }
     return $ret;
 }
 
-sub protocol_class {
+sub selector {
     my $self = shift;
-    my $ret  = $self->{protocol_class};
+    my $ret  = $self->{selector};
     if (@_) {
-        $self->{protocol_class} = shift;
-        $self->{protocol} = $self->_build_protocol;
+        my $obj = shift;
+        my $class = ref $obj;
+        $self->{selector} = $obj;
     }
     return $ret;
 }
 
-sub connect_one {
+sub _connect_one {
     my ($self, $server, $cv) = @_;
 
     return if $self->{_is_connecting}->{$server};
@@ -103,57 +117,68 @@ sub connect_one {
     $port ||= 11211;
 
     $self->{_is_connecting}->{$server} = tcp_connect $host, $port, sub {
-        my ($fh, $host, $port) = @_;
-
-        delete $self->{_is_connecting}->{$server}; # thanks, buddy
-        if (! $fh) {
-            # connect failed
-            warn "failed to connect to $server";
-            if ($self->{auto_reconnect} > $self->{_connect_attempts}->{ $server }++) {
-                # XXX this watcher holds a reference to $self, which means
-                # it will make your program wait for it to fire until 
-                # auto_reconnect attempts have been made. 
-                # if you need to close immediately, you need to call disconnect
-                $self->{_reconnect}->{$server} = AE::timer $self->{reconnect_delay}, 0, sub {
-                    delete $self->{_reconnect}->{$server};
-                    $self->connect_one($server);
-                };
-            }
-        } else {
-            my $h; $h = AnyEvent::Handle->new(
-                fh => $fh,
-                on_drain => sub {
-                    my $h = shift;
-                    if (defined $h->{wbuf} && $h->{wbuf} eq "") {
-                        delete $h->{wbuf}; $h->{wbuf} = "";
-                    }
-                    if (defined $h->{rbuf} && $h->{rbuf} eq "") {
-                        delete $h->{rbuf}; $h->{rbuf} = "";
-                    }
-                },
-                on_eof => sub {
-                    my $h = delete $self->{_server_handles}->{$server};
-                    $h->destroy();
-                    undef $h;
-                },
-                on_error => sub {
-                    my $h = delete $self->{_server_handles}->{$server};
-                    $h->destroy();
-                    $self->connect_one($server) if $self->{auto_reconnect};
-                    undef $h;
-                },
-            );
-
-            push @{$self->{_active_servers}}, $server;
-            $self->{_active_server_count}++;
-            $self->{_server_handles}->{ $server } = $h;
-            delete $self->{_connect_attempts}->{ $server };
-            $self->protocol->prepare_handle( $fh );
-        }
+        $self->_on_tcp_connect($server, @_);
         $cv->end if $cv;
     };
 }
 
+sub _on_tcp_connect {
+    my ($self, $server, $fh, $host, $port) = @_;
+
+    delete $self->{_is_connecting}->{$server}; # thanks, buddy
+    if (! $fh) {
+        # connect failed
+        warn "failed to connect to $server";
+
+        if ($self->{auto_reconnect} > $self->{_connect_attempts}->{ $server }++) {
+            # XXX this watcher holds a reference to $self, which means
+            # it will make your program wait for it to fire until 
+            # auto_reconnect attempts have been made. 
+            # if you need to close immediately, you need to call disconnect
+            $self->{_reconnect}->{$server} = AE::timer $self->{reconnect_delay}, 0, sub {
+                delete $self->{_reconnect}->{$server};
+                $self->_connect_one($server);
+            };
+        }
+    } else {
+        my $h; $h = AnyEvent::Handle->new(
+            fh => $fh,
+            on_drain => sub {
+                my $h = shift;
+                if (defined $h->{wbuf} && $h->{wbuf} eq "") {
+                    delete $h->{wbuf}; $h->{wbuf} = "";
+                }
+                if (defined $h->{rbuf} && $h->{rbuf} eq "") {
+                    delete $h->{rbuf}; $h->{rbuf} = "";
+                }
+            },
+            on_eof => sub {
+                my $h = delete $self->{_server_handles}->{$server};
+                $h->destroy();
+                undef $h;
+            },
+            on_error => sub {
+                my $h = delete $self->{_server_handles}->{$server};
+                $h->destroy();
+                $self->_connect_one($server) if $self->{auto_reconnect};
+                undef $h;
+            },
+        );
+
+        $self->_add_active_server( $server, $h );
+        delete $self->{_connect_attempts}->{ $server };
+        $self->protocol->prepare_handle( $fh );
+    }
+}
+
+sub _add_active_server {
+    my ($self, $server, $h) = @_;
+    push @{$self->{_active_servers}}, $server;
+    $self->{_active_server_count}++;
+    $self->{_server_handles}->{ $server } = $h;
+    $self->{selector}->add_server( $server, $h );
+}
+
 sub connect {
     my $self = shift;
 
@@ -174,11 +199,11 @@ sub connect {
         if (my $cb = $self->{ on_connect }) {
             $cb->($self);
         }
-        $self->drain_queue;
+        $self->_drain_queue;
     };
 
     foreach my $server ( @{ $self->{ servers } }) {
-        $self->connect_one($server, $connect_cv);
+        $self->_connect_one($server, $connect_cv);
     }
 }
 
@@ -186,7 +211,7 @@ sub add {
     my ($self, @args) = @_;
     my $cb = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
     my ($key, $value, $exptime, $noreply) = @args;
-    $self->push_queue( 
+    $self->_push_queue( 
         $self->protocol, 'add', $key, $value, $exptime, $noreply, $cb );
 }
 
@@ -194,71 +219,71 @@ sub decr {
     my ($self, @args) = @_;
     my $cb = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
     my ($key, $value, $initial) = @args;
-    $self->push_queue( $self->protocol, 'decr', $key, $value, $initial, $cb );
+    $self->_push_queue( $self->protocol, 'decr', $key, $value, $initial, $cb );
 }
 
 sub delete {
     my ($self, @args) = @_;
     my $cb       = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
     my $noreply  = !defined $cb;
-    $self->push_queue( $self->protocol, 'delete', @args, $noreply, $cb );
+    $self->_push_queue( $self->protocol, 'delete', @args, $noreply, $cb );
 }
 
 sub get {
     my ($self, $key, $cb) = @_;
-    $self->push_queue( $self->protocol, 'get', $key, $cb );
+    $self->_push_queue( $self->protocol, 'get', $key, $cb );
 }
 
 sub get_handle { shift->{_server_handles}->{ $_[0] } }
 
 sub get_multi {
     my ($self, $keys, $cb) = @_;
-    $self->push_queue( $self->protocol, 'get_multi', $keys, $cb );
+    $self->_push_queue( $self->protocol, 'get_multi', $keys, $cb );
 }
 
 sub incr {
     my ($self, @args) = @_;
     my $cb = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
     my ($key, $value, $initial) = @args;
-    $self->push_queue( $self->protocol, 'incr', $key, $value, $initial, $cb );
+    $self->_push_queue( $self->protocol, 'incr', $key, $value, $initial, $cb );
 }
 
 sub replace {
     my ($self, @args) = @_;
     my $cb = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
     my ($key, $value, $exptime, $noreply) = @args;
-    $self->push_queue( $self->protocol, 'replace', $key, $value, $exptime, $noreply, $cb );
+    $self->_push_queue( $self->protocol, 'replace', $key, $value, $exptime, $noreply, $cb );
 }
 
 sub set {
     my ($self, @args) = @_;
     my $cb = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
     my ($key, $value, $exptime, $noreply) = @args;
-    $self->push_queue( $self->protocol, 'set', $key, $value, $exptime, $noreply, $cb);
+    $self->_push_queue( $self->protocol, 'set', $key, $value, $exptime, $noreply, $cb);
 }
 
 sub append {
     my ($self, @args) = @_;
     my $cb = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
-    $self->push_queue( $self->protocol, 'append', @args, undef, undef, $cb);
+    $self->_push_queue( $self->protocol, 'append', @args, undef, undef, $cb);
 }
 
 sub prepend {
     my ($self, @args) = @_;
     my $cb = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
-    $self->push_queue( $self->protocol, 'prepend', @args, undef, undef, $cb);
+    $self->_push_queue( $self->protocol, 'prepend', @args, undef, undef, $cb);
 }
 
 sub stats {
     my ($self, @args) = @_;
     my $cb = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
     my ($name) = @args;
-    $self->push_queue( $self->protocol, 'stats', $name, $cb );
+    $self->_push_queue( $self->protocol, 'stats', $name, $cb );
 }
 
 sub version {
     my ($self, $cb) = @_;
-    $self->push_queue( $self->protocol, 'version', $cb);
+    $self->_push_queue( $self->protocol, 'version', $cb);
 }
 
 sub flush_all {
@@ -266,16 +291,16 @@ sub flush_all {
     my $cb = pop @args if (ref $args[-1] eq 'CODE' or ref $args[-1] eq 'AnyEvent::CondVar');
     my $noreply = !!$cb;
     my $delay = shift @args || 0;
-    $self->push_queue( $self->protocol, 'flush_all', $delay, $noreply, $cb );
+    $self->_push_queue( $self->protocol, 'flush_all', $delay, $noreply, $cb );
 }
 
-sub push_queue {
+sub _push_queue {
     my ($self, @args) = @_;
     push @{$self->{queue}}, [ @args ];
-    $self->drain_queue unless $self->{_is_draining};
+    $self->_drain_queue unless $self->{_is_draining};
 }
 
-sub drain_queue {
+sub _drain_queue {
     my $self = shift;
     if (! $self->{_is_connected}) {
         if ($self->{_is_connecting}) {
@@ -296,7 +321,7 @@ sub drain_queue {
             my $t; $t = AE::timer 0, 0, sub {
                 $self->{_is_draining}--;
                 undef $t;
-                $self->drain_queue;
+                $self->_drain_queue;
             };
         };
         $proto->$method($guard, $self, @args);
@@ -327,19 +352,11 @@ sub DESTROY {
     $self->disconnect;
 }
     
-sub get_handle_for {
-    my ($self, $key) = @_;
-    my $servers   = $self->{_active_servers};
-    my $i         = $self->{hash_cb}->($key, $self);
-    my $handle    = $self->get_handle( $servers->[ $i ] );
-    if (! $handle) {
-        die "Could not find handle for $key";
-    }
-
-    return $handle;
+sub _get_handle_for {
+    $_[0]->{selector}->get_handle($_[1]);
 }
 
-sub prepare_key {
+sub _prepare_key {
     my ($self, $key) = @_;
     if (my $ns = $self->{namespace}) {
         $key = $ns . $key;
@@ -347,72 +364,55 @@ sub prepare_key {
     return $key;
 }
 
-sub decode_key_value {
-    my ($self, $key, $flags, $data) = @_;
+sub _decode_key_value {
+    my ($self, $key_ref, $flags_ref, $data_ref) = @_;
 
     if (my $ns = $self->{namespace}) {
-        $key =~ s/^$ns//;
+        $$key_ref =~ s/^$ns//;
     }
 
-    if (defined $flags && defined $data) {
-        if ($flags & F_COMPRESS() && HAVE_ZLIB()) {
-            $data = Compress::Zlib::memGunzip($data);
+    if (defined $$flags_ref && defined $$data_ref) {
+        if ($$flags_ref & F_COMPRESS() && HAVE_ZLIB()) {
+            $$data_ref = Compress::Zlib::memGunzip($$data_ref);
         }
-        if ($flags & F_STORABLE()) {
-            $data = Storable::thaw($data);
+        if ($$flags_ref & F_STORABLE()) {
+            $$data_ref = Storable::thaw($$data_ref);
         }
     }
-    return ($key, $data);
+    return ();
 }
 
-sub decode_key {
-    my ($self, $key) = @_;
+sub _prepare_value {
+    my ($self, $cmd, $value_ref, $len_ref, $exptime_ref, $flags_ref) = @_;
 
-    if (my $ns = $self->{namespace}) {
-        $key =~ s/^$ns//;
-    }
-    return $key;
-}
-
-sub prepare_value {
-    my ($self, $cmd, $value, $exptime) = @_;
-
-    my $flags = 0;
-    if (ref $value) {
-        $value = Storable::nfreeze($value);
-        $flags |= F_STORABLE();
+    $$flags_ref = 0;
+    if (ref $$value_ref) {
+        $$value_ref = Storable::nfreeze($$value_ref);
+        $$flags_ref |= F_STORABLE();
     }
 
-    my $len = bytes::length($value);
+    $$len_ref = bytes::length($$value_ref);
     my $threshold = $self->compress_threshold;
     my $compressable = 
         ($cmd ne 'append' && $cmd ne 'prepend') &&
         $threshold && 
         HAVE_ZLIB() &&
-        $len >= $threshold
+        $$len_ref >= $threshold
     ;
+
     if ($compressable) {
-        my $c_val = Compress::Zlib::memGzip($value);
+        my $c_val = Compress::Zlib::memGzip($$value_ref);
         my $c_len = bytes::length($c_val);
 
-        if ($c_len < $len * ( 1 - COMPRESS_SAVINGS() ) ) {
-            $value = $c_val;
-            $len = $c_len;
-            $flags |= F_COMPRESS();
+        if ($c_len < $$len_ref * ( 1 - COMPRESS_SAVINGS() ) ) {
+            $$value_ref = $c_val;
+            $$len_ref   = $c_len;
+            $$flags_ref |= F_COMPRESS();
         }
     }
-    $exptime = int($exptime || 0);
-
-    return ($value, $len, $flags, $exptime);
+    $$exptime_ref = int($$exptime_ref || 0);
 }
 
-sub _modulo_hasher {
-    my ($key, $memcached) = @_;
-    my $count = $memcached->{_active_server_count};
-    return ((String::CRC32::crc32($key) >> 16) & 0x7fff) % $count;
-}
-
-
 1;
 
 __END__
@@ -438,6 +438,21 @@ Cache::Memcached::AnyEvent - AnyEvent Compatible Memcached Client
 
     $memd->disconnect();
 
+    # use ketama algorithm instead of the traditional one
+    my $memd = Cache::Memcached::AnyEvent->new({
+        ...
+        selector_class => 'Ketama',
+        # or, selector => $object 
+    });
+
+    # use binary protocol instead of text
+    my $memd = Cache::Memcached::AnyEvent->new({
+        ...
+        protocol_class => 'Binary',
+        # or, protocol => $object,
+    });
+
+
 =head1 DESRIPTION
 
 WARNING: BETA QUALITY CODE!
@@ -493,16 +508,49 @@ Set to 0 to disable auto-reconnecting
 
 =item compress_threshold => $number
 
-=item hash_cb => $cb->($key, $memcached)
+=item selector => $object
 
-Specify hashing coderef. Callback must return an index to the list of the servers, where C<$key> belongs to.
+The selector is an object responsible for selecting the appropriate
+Memcached server to store a particular key. This object MUST implement
+the following methods:
+
+    $object->add_server( $host_port, $anyevent_handle );
+    my $handle = $object->get_handle( $key );
+
+By default if this argument is not specified, a selector object will
+automatically be created using the value of the C<selector_class>
+argument.
+
+=item selector_class => $class_name_or_fragment
+
+Specifies the selector class to be instantiated. The default value is "Traditional".
+
+If the class name is preceded by a single '+', then that class name with the
+'+' removed will be used as the class name. Otherwise, the prefix
+"Cache::Memcached::AnyEvent::Selector::" will be added to the value
+("Traditional" would be transformed to "Cache::Memcached::AnyEvent::Selector::Traditional")
 
 =item namespace => $namespace
 
 =item procotol => $object
 
+The protocol is an object responsible for handling the actual talking to
+the memcached servers. This object MUST implement all of the memcached
+interface supported by Cache::Memcached::AnyEvent
+
+By default if this argument is not specified, a protocol object will
+automatically be created using the value of the C<protocol_class>
+argument.
+
 =item protocol_class => $classname
 
+Specifies the protocol class to be instantiated. The default value is "Text".
+
+If the class name is preceded by a single '+', then that class name with the
+'+' removed will be used as the class name. Otherwise, the prefix
+"Cache::Memcached::AnyEvent::Protocol::" will be added to the value
+("Text" would be transformed to "Cache::Memcached::AnyEvent::Protocol::Text")
+
 =item reconnect_delay => $seconds
 
 Amount of time to wait between reconnect attempts
@@ -515,6 +563,10 @@ List of servers to use.
 
 C<%args> can also be a hashref.
 
+=head2 auto_reconnect([$bool]);
+
+Get/Set auto_reconnect flag.
+
 =head2 add($key, $value[, $exptime, $noreply], $cb->($rc))
 
 =head2 append($key, $value, $cb->($rc))
@@ -544,8 +596,6 @@ explicitly.
 
 =head2 protocol($object)
 
-=head2 protocol_class($class)
-
 =head2 replace($key, $value[, $exptime, $noreply], $cb->($rc))
 
 =head2 remove($key, $cb->($rc))
@@ -558,7 +608,7 @@ Alias to delete
 
 =head2 stats($cmd, $cb->(\%stats))
 
-=head2 version($cb->(\%result))
+=head2 version( $cb->(\%result) )
 
 =head1 TODO
 
@@ -566,8 +616,6 @@ Alias to delete
 
 =item Binary stats is not yet implemented.
 
-=item Other hashing mechanisms. Consistent Hashing (patches welcome)
-
 =back
 
 =head1 AUTHOR
