@@ -47,4 +47,37 @@ sub get_handle {
     croak "Could not find a suitable handle for key $key";
 }
 
-1;
\ No newline at end of file
+1;
+
+__END__
+
+=head1 NAME
+
+Cache::Memcached::AnyEvent::Selector::Ketama - Ketama Server Selection Algorithm 
+=head1 SYNOPSIS
+
+    use Cache::Memcached::AnyEvent;
+    my $memd = Cache::Memcached::AnyEvent->new({
+        ...
+        selector_class => 'Ketama',
+    });
+
+=head1 DESCRIPTION
+
+Implements the ketama server selection mechanism, 
+
+=head1 METHODS
+
+=head2 $class->new( memcached => $memd )
+
+Constructor.
+
+=head2 $selector->add_server( $server, $handle )
+
+Called when a new server connection is made.
+
+=head2 $handle = $selector->get_handle( $key )
+
+Returns the AnyEvent handle that is responsible for handling C<$key>
+
+=cut
